﻿'*******************************************************************
' Program: Data Center Visio Rack Tool
' Author: Albert E Edlund
' Date:
'
' Purpose:
' Operating system may be an attribute of either a component or a virtual host
'
'
'*******************************************************************



Imports Microsoft.Practices.EnterpriseLibrary.Common
Imports Microsoft.Practices.EnterpriseLibrary.Data
Imports Microsoft.Practices.EnterpriseLibrary.ExceptionHandling
Imports Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging



Imports System
Imports System.ComponentModel
Imports System.Collections.Generic

Imports System.Collections.ObjectModel
Imports System.Data.Common
Imports System.Data

Imports System.IO
Imports System.Xml
Imports System.Collections
Imports System.Runtime.InteropServices


Imports Microsoft.Office.Interop.Visio


Imports DC_Rack.Support.clsVisioSupport
Imports DC_Rack.Objects.clsDC_RackData

Public Class clsAttrOperatingSystem


#Region " properties and definitions "

    Public m_db As Database
    Public m_ds As DataSet
    Public m_dt As DataTable


    Public m_dc As dc_System
    Public m_oc As Collection(Of dc_System)

    Private strUserId As String

    Public m_deleteCommand As DbCommand
    Public m_insertCommand As DbCommand
    Public m_updateCommand As DbCommand

    Private _clsVisio As DC_Rack.Support.clsVisioSupport

    Public Const readAll As String = "selectTblOperatingSystem"

    ' who am I
    Private myEnum As DC_RackTables = DC_RackTables.tblAttrOperatingSystem

    Private m_visshape As Microsoft.Office.Interop.Visio.Shape
    Public Property _VisShape() As Microsoft.Office.Interop.Visio.Shape
        Get
            Return Me.m_visshape
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Shape)
            m_visshape = value
        End Set
    End Property


    '********* visio snippet vb.net vis0041vb ******************
    Private _drawingApplication As Microsoft.Office.Interop.Visio.Application
    Public Property drawingApplication() As Microsoft.Office.Interop.Visio.Application
        Get
            Return Me._drawingApplication
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Application)
            If (value Is Nothing) Then
                Throw New System.ArgumentNullException("ApplicationValueIsNull", _
                    "drawingApplication is Null.")
            End If
            Me._drawingApplication = value
        End Set
    End Property

    '********* visio snippet vb.net vis0043vb ******************
    Private _drawingDocument As Microsoft.Office.Interop.Visio.Document
    Public Property drawingDocument() As Microsoft.Office.Interop.Visio.Document
        Get
            Return Me._drawingDocument
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Document)
            If (value Is Nothing) Then
                Throw New System.ArgumentNullException("DocumentValueIsNull", _
                    "drawingDocument is Null.")
            End If
            Me._drawingDocument = value
        End Set
    End Property

    '********* visio snippet vb.net vis0045vb ******************
    Private _drawingWindow As Microsoft.Office.Interop.Visio.Window
    Public Property drawingWindow() As Microsoft.Office.Interop.Visio.Window
        Get
            Return Me._drawingWindow
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Window)
            If (value Is Nothing) Then
                Throw New System.ArgumentNullException("WindowValueIsNull", _
                    "drawingWindow is Null.")
            End If
            Me._drawingWindow = value
        End Set
    End Property

    '********* visio snippet vb.net vis0044vb ******************
    Private _drawingPage As Microsoft.Office.Interop.Visio.Page
    Public Property drawingPage() As Microsoft.Office.Interop.Visio.Page
        Get
            Return Me._drawingPage
        End Get
        Set(ByVal value As Microsoft.Office.Interop.Visio.Page)
            If (value Is Nothing) Then
                Throw New System.ArgumentNullException("PageValueIsNull", _
                    "drawingPage is Null.")
            End If
            Me._drawingPage = value
        End Set
    End Property

    Public m_customstencilpath As String = My.Settings.path_custom_stencils
    Public Property _CustomStencilPath() As String
        Get
            Return Me.m_customstencilpath
        End Get
        Set(ByVal value As String)
            Me.m_customstencilpath = value
        End Set
    End Property



#End Region

#Region " Move/Transform Data "


    '
    ' create columns
    '
    Public Sub CreateTableColumns(ByRef tempTable As DataTable)

        Dim newDC As dc_System = New dc_System(strUserId)

        Try

            Dim colOsKey As New System.Data.DataColumn
            colOsKey.ColumnName = "osKey"
            colOsKey.DataType = GetType(Int32)
            colOsKey.DefaultValue = m_dc.osKey
            tempTable.Columns.Add(colOsKey)

            Dim colOsCompKey As New System.Data.DataColumn
            colOsCompKey.ColumnName = "osCompKey"
            colOsCompKey.DataType = GetType(Int32)
            colOsCompKey.DefaultValue = m_dc.osCompKey
            tempTable.Columns.Add(colOsCompKey)

            Dim colOsVirtualHostKey As New System.Data.DataColumn
            colOsVirtualHostKey.ColumnName = "osVirtualHostKey"
            colOsVirtualHostKey.DataType = GetType(Int32)
            colOsVirtualHostKey.DefaultValue = m_dc.osVirtualHostKey
            tempTable.Columns.Add(colOsVirtualHostKey)

            Dim colOsSystemName As New System.Data.DataColumn
            colOsSystemName.ColumnName = "osSystemName"
            colOsSystemName.DataType = GetType(String)
            colOsSystemName.DefaultValue = m_dc.osSystemName
            tempTable.Columns.Add(colOsSystemName)

            Dim colOsVendor As New System.Data.DataColumn
            colOsVendor.ColumnName = "osVendor"
            colOsVendor.DataType = GetType(String)
            colOsVendor.DefaultValue = m_dc.osVendor
            tempTable.Columns.Add(colOsVendor)

            Dim colOsOperatingSystem As New System.Data.DataColumn
            colOsOperatingSystem.ColumnName = "osOperatingSystem"
            colOsOperatingSystem.DataType = GetType(String)
            colOsOperatingSystem.DefaultValue = m_dc.osOperatingSystem
            tempTable.Columns.Add(colOsOperatingSystem)

            Dim colOsVersion As New System.Data.DataColumn
            colOsVersion.ColumnName = "osVersion"
            colOsVersion.DataType = GetType(String)
            colOsVersion.DefaultValue = m_dc.osVersion
            tempTable.Columns.Add(colOsVersion)

            Dim colOsPlatform3264 As New System.Data.DataColumn
            colOsPlatform3264.ColumnName = "osPlatform3264"
            colOsPlatform3264.DataType = GetType(String)
            colOsPlatform3264.DefaultValue = m_dc.osPlatform3264
            tempTable.Columns.Add(colOsPlatform3264)

            Dim colOsServicePack As New System.Data.DataColumn
            colOsServicePack.ColumnName = "OsServicePack"
            colOsServicePack.DataType = GetType(String)
            colOsServicePack.DefaultValue = m_dc.OsServicePack
            tempTable.Columns.Add(colOsServicePack)

            Dim colOsFixLevel As New System.Data.DataColumn
            colOsFixLevel.ColumnName = "osFixLevel"
            colOsFixLevel.DataType = GetType(String)
            colOsFixLevel.DefaultValue = m_dc.osFixLevel
            tempTable.Columns.Add(colOsFixLevel)

            Dim colOsComments As New System.Data.DataColumn
            colOsComments.ColumnName = "osComments"
            colOsComments.DataType = GetType(String)
            colOsComments.DefaultValue = m_dc.osComments
            tempTable.Columns.Add(colOsComments)

            Dim colOsLastEdit As New System.Data.DataColumn
            colOsLastEdit.ColumnName = "osLastEdit"
            colOsLastEdit.DataType = GetType(String)
            colOsLastEdit.DefaultValue = m_dc.osLastEdit
            tempTable.Columns.Add(colOsLastEdit)

            Dim colOsLastUpdate As New System.Data.DataColumn
            colOsLastUpdate.ColumnName = "osLastUpdate"
            colOsLastUpdate.DataType = GetType(Date)
            colOsLastUpdate.DefaultValue = m_dc.osLastUpdate
            tempTable.Columns.Add(colOsLastUpdate)

            newDC = Nothing

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

        If newDC IsNot Nothing Then
            newDC = Nothing
        End If

    End Sub


    '
    ' use for writing records
    '
    Public Sub movePropertiesToDataRow _
                (ByRef tempRow As DataRow, _
                 ByVal tempDC As dc_System)

        Try
            With tempDC
                tempRow.SetField("osKey", .osKey)
                tempRow.SetField("osCompKey", .osCompKey)
                tempRow.SetField("osVirtualHostKey", .osVirtualHostKey)
                tempRow.SetField("osSystemName", .osSystemName)
                tempRow.SetField("osVendor", .osVendor)
                tempRow.SetField("osOperatingSystem", .osOperatingSystem)
                tempRow.SetField("osVersion", .osVersion)
                tempRow.SetField("osPlatform3264", .osPlatform3264)
                tempRow.SetField("OsServicePack", .OsServicePack)
                tempRow.SetField("osFixLevel", .osFixLevel)
                tempRow.SetField("osComments", .osComments)
                tempRow.SetField("osLastEdit", strUserId)
                tempRow.SetField("osLastUpdate", .osLastUpdate)
            End With

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub


    '
    ' for reading records
    '
    Public Sub moveDataRowToProperties _
            (ByVal tempRow As DataRow)

        Try
            With m_dc
                .initProperties()
                .osKey = tempRow.Item("osKey")
                .osCompKey = tempRow.Item("osCompKey")
                .osVirtualHostKey = tempRow.Item("osVirtualHostKey")
                .osVendor = tempRow.Item("osVendor")
                .osOperatingSystem = tempRow.Item("osOperatingSystem")
                .osVersion = tempRow.Item("osVersion")
                .osPlatform3264 = tempRow.Item("osPlatform3264")
                .OsServicePack = tempRow.Item("OsServicePack")
                .osFixLevel = tempRow.Item("osFixLevel")
                .osComments = tempRow.Item("osComments")
                .osLastEdit = tempRow.Item("osLastEdit")
                .osLastUpdate = tempRow.Item("osLastUpdate")
            End With

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    ' for reading records
    '
    Public Sub moveDataRowToProperties _
                (ByVal tempRow As DataRow, _
                ByRef tempDC As dc_System)

        Try
            With tempDC
                .initProperties()
                .osKey = tempRow.Item("osKey")
                .osCompKey = tempRow.Item("osCompKey")
                .osVirtualHostKey = tempRow.Item("osVirtualHostKey")
                .osVendor = tempRow.Item("osVendor")
                .osOperatingSystem = tempRow.Item("osOperatingSystem")
                .osVersion = tempRow.Item("osVersion")
                .osPlatform3264 = tempRow.Item("osPlatform3264")
                .OsServicePack = tempRow.Item("OsServicePack")
                .osFixLevel = tempRow.Item("osFixLevel")
                .osComments = tempRow.Item("osComments")
                .osLastEdit = tempRow.Item("osLastEdit")
                .osLastUpdate = tempRow.Item("osLastUpdate")
            End With

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Sub moveDataTableToObservableCollection _
            (ByVal dtTemp As DataTable)

        Try

            If 0 < dtTemp.Rows.Count Then

                If m_oc IsNot Nothing Then
                    m_oc = Nothing
                    m_oc = New Collection(Of dc_System)
                Else
                    m_oc = New Collection(Of dc_System)
                End If

                Dim drTemp As DataRow
                Dim dcTemp As dc_System
                For Each drTemp In dtTemp.Rows
                    dcTemp = New dc_System
                    moveDataRowToProperties(drTemp, dcTemp)
                    m_oc.Add(dcTemp)
                Next

            End If

        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Function moveDataTableToXML() As XmlDocument

        Dim writer = New System.IO.StringWriter
        Dim m_dt_xml As Xml.XmlDocument = Nothing

        Try


            m_dt.WriteXml(writer, XmlWriteMode.WriteSchema, True)
            m_dt_xml = Nothing
            m_dt_xml = New Xml.XmlDocument
            m_dt_xml.LoadXml(writer.ToString)

        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        Finally

            writer = Nothing

        End Try

        Return m_dt_xml


    End Function



#End Region

#Region " standard data "

    '
    '
    '
    Public Sub initDataSetAndTable()

        Try

            ' make sure the root stuff is there
            If m_ds Is Nothing Then
                m_ds = New DataSet
            Else
                m_ds = Nothing
                m_ds = New DataSet
            End If
            m_ds.SchemaSerializationMode = SchemaSerializationMode.IncludeSchema

            If m_dt Is Nothing Then
                m_dt = New DataTable
            Else
                m_dt = Nothing
                m_dt = New DataTable
            End If

            If m_oc Is Nothing Then
                m_oc = New Collection(Of dc_System)
            Else
                m_oc = Nothing
                m_oc = New Collection(Of dc_System)
            End If

            If m_dc IsNot Nothing Then
                m_dc = Nothing
                m_dc = New dc_System(strUserId)
            Else
                m_dc = New dc_System(strUserId)
            End If

            CreateTableColumns(m_dt)
            m_dt.TableName = dc_racktables.tblattroperatingsystem.tostring
            m_ds.Tables.Add(m_dt)

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    ' create a record based on property values move it to the generic datatable
    ' then copy the table to the database
    '
    Public Function addAttrOperatingSystemDataSourceRecord _
        (ByVal tempDC As dc_System, _
         Optional ByVal blnSilent As Boolean = False) _
        As Integer

        Dim intRowsAffected As Integer = 0

        Try

            initDataSetAndTable()

            Dim m_dr As DataRow = m_ds.Tables(dc_racktables.tblattroperatingsystem.tostring).NewRow
            movePropertiesToDataRow(m_dr, tempDC)
            m_ds.Tables(dc_racktables.tblattroperatingsystem.tostring).Rows.Add(m_dr)
            intRowsAffected = dbUpdateDataSet()


            If blnSilent = False Then
                MsgBox("DataRowsUpated = " & intRowsAffected)
            End If

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

        Return intRowsAffected

    End Function

    '
    '
    '
    Private Sub buildInsertOSCommand()

        Try
            m_insertCommand = m_db.GetStoredProcCommand("insertAttrOperatingSystem")

            Dim cmdActive As DbCommand
            cmdActive = m_insertCommand
            m_db.AddInParameter(cmdActive, "osCompKey", DbType.Int32, "osCompKey", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osVirtualHostKey", DbType.Int32, "osVirtualHostKey", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osSystemName", DbType.String, "osSystemName", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osVendor", DbType.String, "osVendor", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osOperatingSystem", DbType.String, "osOperatingSystem", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osVersion", DbType.String, "osVersion", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osPlatform3264", DbType.String, "osPlatform3264", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "OsServicePack", DbType.String, "OsServicePack", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "OsComments", DbType.String, "OsComments", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osFixLevel", DbType.String, "osFixLevel", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osLastEdit", DbType.String, "osLastEdit", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osLastUpdate", DbType.Date, "osLastUpdate", DataRowVersion.Current)


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Private Sub buildUpdateOSCommand()

        Try
            m_updateCommand = m_db.GetStoredProcCommand("updateAttrOperatingSystem")

            Dim cmdActive As DbCommand
            cmdActive = m_updateCommand
            m_db.AddInParameter(cmdActive, "osKey", DbType.Int32, "osKey", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osCompKey", DbType.Int32, "osCompKey", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osVirtualHostKey", DbType.Int32, "osVirtualHostKey", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osSystemName", DbType.String, "osSystemName", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osVendor", DbType.String, "osVendor", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osOperatingSystem", DbType.String, "osOperatingSystem", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osVersion", DbType.String, "osVersion", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osPlatform3264", DbType.String, "osPlatform3264", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "OsServicePack", DbType.String, "OsServicePack", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osFixLevel", DbType.String, "osFixLevel", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "OsComments", DbType.String, "OsComments", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osLastEdit", DbType.String, "osLastEdit", DataRowVersion.Current)
            m_db.AddInParameter(cmdActive, "osLastUpdate", DbType.Date, "osLastUpdate", DataRowVersion.Current)


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Private Sub buildDeleteOsCommand()

        Try
            m_deleteCommand = m_db.GetStoredProcCommand("deleteAttrOperatingSystem")

            Dim cmdActive As DbCommand
            cmdActive = m_deleteCommand
            m_db.AddInParameter(cmdActive, "osKey", DbType.Int32, "osKey", DataRowVersion.Current)


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Sub dbSelectAllOperatingSystems()


        Try

            If m_dc Is Nothing Then
                m_dc = New dc_System
            Else
                m_dc = Nothing
                m_dc = New dc_System
            End If

            initDataSetAndTable()

            ' Retrieve the initial data
            Dim cmd As DbCommand = m_db.GetStoredProcCommand(readAll)
            m_db.LoadDataSet(cmd, m_ds, dc_racktables.tblattroperatingsystem.tostring)
            m_dt = m_ds.Tables(dc_racktables.tblattroperatingsystem.tostring)


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Function dbUpdateDataSet() As Int16

        Dim rowsAffected As Int16 = 0

        Try

            rowsAffected = m_db.UpdateDataSet(m_ds, dc_racktables.tblattroperatingsystem.tostring, _
                                        m_insertCommand, _
                                        m_updateCommand, _
                                        m_deleteCommand, _
                                        UpdateBehavior.Standard)
            'msgbox rowsAffected

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If

        End Try

        Return rowsAffected

    End Function




#End Region

#Region " database calls "

    '
    '
    '
    Public Sub dbSelectAttrOperatingSystemByRowKey _
            (ByVal lngRowKey As Int32)



        Try

            initDataSetAndTable()

            ' Retrieve the initial data
            Dim cmd As DbCommand = m_db.GetStoredProcCommand("selectAttrOperatingSystemByRowKey")
            m_db.AddInParameter(cmd, "@RowKey", DbType.Int32, lngRowKey)
            m_db.LoadDataSet(cmd, m_ds, dc_racktables.tblattroperatingsystem.tostring)
            m_dt = m_ds.Tables(dc_racktables.tblattroperatingsystem.tostring)


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch Ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(Ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Sub dbSelectAttrOperatingSystemByUrriKeys _
        (ByVal colURRIKeys As Collection(Of clsDC_RackData.rackURRI))

        initDataSetAndTable()


        Dim dtInput As DataTable = New DataTable
        Dim dtOutput As DataTable = New DataTable
        Dim drTemp As DataRow
        Dim lngRackKey As Int32
        Dim colRacks As Collection = New Collection
        Dim rckUrri As clsDC_RackData.rackURRI

        For Each rckURRI In colURRIKeys

            colRacks.Add(rckUrri.rckKey)
        Next rckURRI

        Try

            For intX = 1 To colRacks.Count
                lngRackKey = colRacks(intX)
                If Not (dtInput Is Nothing) Then
                    dtInput = Nothing
                    dtInput = New DataTable
                Else
                    dtInput = New DataTable
                End If
                dtInput = GetAttrOperatingSystemByRack(lngRackKey)
                With dtInput
                    If intX = 1 Then
                        dtOutput = Nothing
                        dtOutput = New DataTable
                        dtOutput = dtInput.Copy
                    Else
                        For Each drTemp In .Rows
                            dtOutput.ImportRow(drTemp)
                        Next
                    End If
                End With
            Next intX

            For Each drTemp In dtOutput.Rows
                m_ds.Tables(DC_RackTables.tblAttrOperatingSystem.ToString).ImportRow(drTemp)
            Next

            m_dt = m_ds.Tables(DC_RackTables.tblAttrOperatingSystem.ToString)
            m_dt.TableName = DC_RackTables.tblAttrOperatingSystem.ToString


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        Catch Ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(Ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Function GetAttrOperatingSystemByRack _
                (ByVal lngRackKey As Int32) _
                As DataTable

        Dim m_dt1 As DataTable = New DataTable
        Dim dtReturn As DataTable = New DataTable
        Dim m_ds1 As DataSet = New DataSet

        Try

            Dim cmd1 As DbCommand
            cmd1 = m_db.GetStoredProcCommand("selectAttrOperatingSystemByRack")
            m_db.AddInParameter(cmd1, "@compKey", DbType.Int32, lngRackKey)
            m_db.LoadDataSet(cmd1, m_ds1, "temp")
            m_dt1 = m_ds1.Tables("temp")

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
            m_dt1 = Nothing
        End Try

        GetAttrOperatingSystemByRack = m_dt1

    End Function

    '
    '
    '
    Public Sub dbUpdateOSResetComponentKey(ByVal intCompKey As Int32)



        If intCompKey = -1 Then Exit Sub

        Try

            Dim cmd As DbCommand = m_db.GetStoredProcCommand("updateOSResetComponentKey")
            m_db.AddInParameter(cmd, "@compKey", DbType.Int32, intCompKey)

            Dim affectedRows As Int16 = m_db.ExecuteNonQuery(cmd)

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try


    End Sub


    '
    '
    '
    Public Sub dbSelectAttrOperatingSystemByCompKey(ByVal intCompKey As Int32)


        Try


            initDataSetAndTable()

            ' Retrieve the initial data
            Dim cmd As DbCommand = m_db.GetStoredProcCommand("selectAttrOperatingSystemByCompKey")
            m_db.AddInParameter(cmd, "@compKey", DbType.Int32, intCompKey)
            m_db.LoadDataSet(cmd, m_ds, dc_racktables.tblattroperatingsystem.tostring)

            m_dt = m_ds.Tables(dc_racktables.tblattroperatingsystem.tostring)

            If m_dt.Rows.Count = 1 Then
                Dim m_dr As System.Data.DataRow = m_dt.Rows(0)
                moveDataRowToProperties(m_dr)
            End If


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Sub dbSelectAttrOperatingSystemByVHKey(ByVal intVHKey As Int32)


        Try


            initDataSetAndTable()

            ' Retrieve the initial data
            Dim cmd As DbCommand = m_db.GetStoredProcCommand("selectAttrOperatingSystemByVHKey")
            m_db.AddInParameter(cmd, "@VHKey", DbType.Int32, intVHKey)
            m_db.LoadDataSet(cmd, m_ds, dc_racktables.tblattroperatingsystem.tostring)

            m_dt = m_ds.Tables(dc_racktables.tblattroperatingsystem.tostring)

            If m_dt.Rows.Count = 1 Then
                Dim m_dr As System.Data.DataRow = m_dt.Rows(0)
                moveDataRowToProperties(m_dr)
            End If


        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    '
    '
    '
    Public Sub dbDeleteAttrOperatingSystemByCompKey(ByVal intCompKey As Int32)


        Try



            ' Retrieve the initial data
            Dim cmd As DbCommand = m_db.GetStoredProcCommand("deleteAttrOperatingSystemByCompKey")
            m_db.AddInParameter(cmd, "@compKey", DbType.Int32, intCompKey)


            Dim affectedRows As Int16 = m_db.ExecuteNonQuery(cmd)

        Catch sql As SqlException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(sql, "Database Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub




#End Region

#Region " constructor "


    Public Sub New(ByVal visApp As Microsoft.Office.Interop.Visio.Application, _
                   ByVal visDoc As Microsoft.Office.Interop.Visio.Document)
        Try


            m_db = DatabaseFactory.CreateDatabase()

            strUserId = Environment.UserDomainName & "/" & Environment.UserName

            '
            ' init an object class so that the default constants are available
            ' to the table definitions
            '

            If m_dc Is Nothing Then
                m_dc = New dc_System
            Else
                m_dc = Nothing
                m_dc = New dc_System
            End If

            initDataSetAndTable()


            _drawingApplication = visApp
            _drawingDocument = visDoc


            _clsVisio = New DC_Rack.Support.clsVisioSupport

            _clsVisio.drawingApplication = visApp
            _clsVisio.drawingDocument = visDoc

            m_updateCommand = Nothing
            m_insertCommand = Nothing
            m_deleteCommand = Nothing

            buildInsertOSCommand()
            buildDeleteOsCommand()
            buildUpdateOSCommand()


        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try




    End Sub

#End Region

#Region " Visio Attributes "

    '
    '
    '
    Public Sub addTestOSProperties()

        With m_dc
            .osKey = -1
            .osCompKey = -1
            .osVirtualHostKey = -1
            .osSystemName = "ServerOne"
            .osVendor = "Microsoft"
            .osVersion = "Core"
            .osPlatform3264 = "x64"
            .osOperatingSystem = "ws2k8r2"
            .OsServicePack = "SP1"
            .osFixLevel = "1.0"
        End With


    End Sub

    '
    'TODO: change service level to service pack, add osrelease
    '
    Public Sub UpdateAttrOperatingSystemProperties _
            (ByRef visShape As Microsoft.Office.Interop.Visio.Shape, _
             ByVal drOS As System.Data.DataRow)


        Try


            With m_dc
                If Not (drOS Is Nothing) Then

                    moveDataRowToProperties(drOS)

                Else

                    addTestOSProperties()

                End If


                _clsVisio.ForceShapeData(visShape, "osKey", .osKey, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osCompKey", .osCompKey, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osVirtualHostKey", .osVirtualHostKey, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osSystemName", .osSystemName, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osVendor", .osVendor, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osVersion", .osVersion, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osPlatform3264", .osPlatform3264, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osOperatingSystem", .osOperatingSystem, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osServicePack", .OsServicePack, "attrOS")
                _clsVisio.ForceShapeData(visShape, "osFixLevel", .osFixLevel, "attrOS")


            End With



        Catch com As COMException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(com, "Com+ Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try

    End Sub

    Private Const constOSPinX As String = "1.5 in"
    Private Const constOSPinY As String = "6.5 in"
    Private Const constOSAlign As VisListAlignment = VisListAlignment.visListAlignLeftOrTop
    Private Const constOSDir As VisListDirection = VisListDirection.visListDirTopToBottom
    Private Const constOSHeader As String = "Operating System"
    Private Const constOSMaster As String = "Container 4"

    '
    '
    '
    Public Function buildComponentOperatingSystemContainer _
                        (ByVal intCompKey As Int32, _
                       ByVal _clsData As clsDC_RackData) _
                        As Microsoft.Office.Interop.Visio.Shape

        Dim shpContainer As Microsoft.Office.Interop.Visio.Shape = Nothing

        ' just to make sure
        _clsVisio.drawingPage = drawingPage

        Try


            Dim colInput As Collection = New Collection
            Dim drTemp As System.Data.DataRow = Nothing
            Dim operatingsystemQuery = From operatingsystem In _clsData.m_ds.Tables(DC_RackTables.tblAttrOperatingSystem.ToString).AsEnumerable() _
                       Where operatingsystem!oscompkey = intCompKey
                      Select operatingsystem

            If operatingsystemQuery.Count = 0 Then Return shpContainer

            ' should only be a single operating system 
            If operatingsystemQuery.Count = 1 Then
                For Each o In operatingsystemQuery
                    colInput.Add("osKey_" & o.Item("osKey").ToString)
                    'System.Diagnostics.Debug.WriteLine(o.Item("osSystemName").ToString)
                    'System.Diagnostics.Debug.WriteLine(o.Item("osKey").ToString)
                    drTemp = o
                Next o
            End If

            ' create a list control
            Dim listControl As Microsoft.Office.Interop.Visio.Shape = Nothing
            ' pass in the created collection and return a populated list control
            Dim colMembers As Collection = New Collection

            Dim strStencil As String = ""
            Dim strMaster As String = ""
            Dim strDGMaster As String = ""
            Dim visDG As Microsoft.Office.Interop.Visio.Master = Nothing
            Dim intLabelFont As Integer
            Dim intValueFont As Integer
            Dim dblCalloutWidth As Double
            Dim dgmasterQuery = From datagraphic In _clsData.m_ds.Tables(DC_RackTables.tblDGMasters.ToString).AsEnumerable() _
                                Where datagraphic!dgmTable = DC_RackTables.tblAttrOperatingSystem.ToString _
                                Select datagraphic

            ' should only be one..
            For Each m In dgmasterQuery
                If LCase(m!dgmTable) = LCase(DC_RackTables.tblAttrOperatingSystem.ToString) Then
                    strStencil = m!dgmSurrogateStencil
                    strStencil = _CustomStencilPath & strStencil
                    strMaster = m!dgmSurrogateMaster
                    intLabelFont = CInt(m!dgmFontLabel)
                    intValueFont = CInt(m!dgmFontValue)
                    dblCalloutWidth = CDbl(m!dgmCalloutWidth)
                    strDGMaster = m!dgmMaster
                    Exit For
                End If
            Next


            colMembers = _clsVisio.dropDropDownBoxItems(colInput, strMaster, strStencil, "1.5in", "1in", vbTrue)

            Dim visShape As Microsoft.Office.Interop.Visio.Shape = Nothing
            Dim visCell As Microsoft.Office.Interop.Visio.Cell = Nothing

            Dim strName As String = ""
            Dim strKey As String = ""
            For Each shapeId In colMembers
                visShape = drawingPage.Shapes.ItemFromID(shapeId)

                If visShape.CellExists("user.attrkey", False) = True Then
                    strName = visShape.Cells("user.attrkey").ResultStr("")
                Else
                    strName = ""
                    GoTo NextShape
                End If

                ' clear any shape data properties
                _clsVisio.deleteShapeData(visShape)

                ' set it semi-transparent, the surrogate shape is pretty busy
                Dim strTrans As String = "35%"
                ' first the fill properties
                _clsVisio.recurseApplyCellFormula(visShape, VisSectionIndices.visSectionObject, VisRowIndices.visRowFill, VisCellIndices.visFillForegndTrans, strTrans)
                _clsVisio.recurseApplyCellFormula(visShape, VisSectionIndices.visSectionObject, VisRowIndices.visRowFill, VisCellIndices.visFillBkgndTrans, strTrans)

                _clsVisio.recurseApplyCellFormula(visShape, VisSectionIndices.visSectionObject, VisRowIndices.visRowFill, VisCellIndices.visFillForegnd, "THEMEGUARD(MSOTINT(THEME(""FillColor""),80))")

                ' then the line properties
                _clsVisio.recurseApplyCellFormula(visShape, VisSectionIndices.visSectionObject, VisRowIndices.visRowLine, VisCellIndices.visLineColorTrans, strTrans)
                ' prep it for applying the datagraphics by applying the data from the dgmasters table
                _clsVisio.addDataGraphicUserFieldsToShape _
                        (visShape, _
                        intValueFont.ToString, _
                        intLabelFont.ToString, _
                        dblCalloutWidth.ToString)

                'Check that the requested Data Graphic Master exits
                For Each visMaster In drawingDocument.Masters
                    If visMaster.Type = VisMasterTypes.visTypeDataGraphic Then
                        If visMaster.Name = strDGMaster Then
                            visDG = visMaster
                            Exit For
                        End If
                    End If
                Next

                ' go read the current records
                For Each o In operatingsystemQuery
                    strKey = "osKey_" & o.Item("osKey").ToString
                    If strKey = strName Then
                        UpdateAttrOperatingSystemProperties(visShape, o)
                        visShape.Text = o.Item("osSystemName").ToString

                        ' move the name to comment field so we get mouse over
                        If visShape.CellsSRCExists(VisSectionIndices.visSectionObject, VisRowIndices.visRowMisc, VisCellIndices.visComment, False) = True Then
                            visCell = visShape.CellsSRC(VisSectionIndices.visSectionObject, VisRowIndices.visRowMisc, VisCellIndices.visComment)
                            _clsVisio.SetCellValueToString(visCell, strName)
                        End If
                        ' allow the legend to keep track of the shapes
                        _clsVisio.ForceUserProperty(visShape, "visLegendShape", "2.0")

                        'Apply the Data Graphic to the Shape
                        visShape.DataGraphic = visDG

                        ' unlock the groups so we can change the font sizes
                        _clsVisio.recurseApplyCellFormula(visShape, VisSectionIndices.visSectionObject, VisRowIndices.visRowLock, VisCellIndices.visLockGroup, "0")

                        If visShape.CellExists("user.dbIntLabelFont", False) = True Then

                            ' get the loaded values 
                            intLabelFont = CInt(visShape.Cells("user.dbintlabelfont").ResultIU)
                            intValueFont = CInt(visShape.Cells("user.dbintvaluefont").ResultIU)
                            dblCalloutWidth = CDbl(visShape.Cells("user.dbdblcallout").ResultIU)

                            ' set the callout width = 85% of the surrogate shapes width (as a formula) so that it fits inside the shape
                            Dim strId As String = visShape.NameID & "!Width*.85"
                            strId = "Guard(" & strId & ")"

                            _clsVisio.recurseApplyCustomFormula(visShape, "prop.msvcalloutproplabelsize", intLabelFont.ToString & " pt")
                            _clsVisio.recurseApplyCustomFormula(visShape, "prop.msvcalloutpropvaluesize", intValueFont.ToString & " pt")
                            _clsVisio.recurseApplyCustomFormula(visShape, "prop.msvcalloutpropwidth", strId)

                        End If

                        Exit For
                    End If
                Next o
NextShape:
            Next shapeId

            shpContainer = _clsVisio.addContainerToPage(constOSMaster, "contOperatingSystem", "")

            listControl = _clsVisio.loadListControl(colMembers, constOSAlign, constOSDir)
            listControl.Name = "listOS"
            listControl.NameU = "listOS"

            ' add the list control to the container
            _clsVisio.AddShapeToContainer(shpContainer, listControl)

            shpContainer.Cells("pinx").Formula = constOSPinX
            shpContainer.Cells("piny").Formula = constOSPinY


        Catch com As COMException
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(com, "Com+ Policy")
            If (rethrow) Then
                Throw
            End If
        Catch ex As Exception
            Dim rethrow As Boolean = ExceptionPolicy.HandleException(ex, "Log Only Policy")
            If (rethrow) Then
                Throw
            End If
        End Try
        System.Windows.Forms.Application.DoEvents()
        Return shpContainer

    End Function

#End Region



End Class
